import { __assign, __spreadArrays } from "tslib";
import { MOUNT_MODEL, INITIALIZE_STATE, MARK_VALIDATED, SET_VALUE, SHOW_ERRORS, } from '../action-types';
import { fromDotProp, getIn, isNumber, isObject, isArray } from '../../utils';
var constructBlueprint = function (arr, state) {
    var _a;
    if (state === void 0) { state = {}; }
    if (arr.length == 0) {
        return undefined;
    }
    else if (arr[0] === "*" || isNumber(arr[0])) {
        return __assign(__assign({}, state), { '*': {
                length: Math.max(isNumber(arr[0]) ? parseInt(arr[0]) + 1 : 0, getIn(state, '*', 'length') || 0),
                value: constructBlueprint(arr.slice(1), getIn(state, '*', 'value')),
            } });
    }
    else {
        if ('*' in state) {
            if (process.env.NODE_ENV !== 'production') {
                console.error("Usage Error!\nYou're mixing Arrays and Objects within the same model value");
            }
        }
        return __assign(__assign({}, state), (_a = {}, _a[arr[0]] = constructBlueprint(arr.slice(1), state[arr[0]]), _a));
    }
};
var escapeRegExp = function (str) {
    return str.replace(/[.*]/g, '\\$&');
};
var findBestMatch = function (map, match) {
    var regex = escapeRegExp("^(" + match.replace(/\.(\d+)\b/g, '.($1|*))?(') + ")?$");
    var matches = Object.keys(map).reduce(function (acc, key) {
        var _a;
        if (key.match(regex)) {
            return __assign(__assign({}, acc), (_a = {}, _a[key] = map[key], _a));
        }
        return acc;
    }, {});
    // parse result
    var keys = Object.keys(matches);
    if (keys.length === 0) {
        return undefined;
    }
    var index = -2;
    return keys.reduce(function (acc, k) {
        var lastDigit = k.match(/\d+(?![\s\S]*\.\d+)/);
        if (lastDigit) {
            var lastIndex = k.split('.').lastIndexOf(lastDigit.toString());
            if (lastIndex > index) {
                index = lastIndex;
                return matches[k];
            }
        }
        return acc;
    }, matches[keys[0]]);
};
var initializeState = function (state, initialState, currentPath) {
    if (state === void 0) { state = {}; }
    if (currentPath === void 0) { currentPath = ''; }
    var keys = Object.keys(state);
    if (keys.length === 0) {
        return findBestMatch(initialState, currentPath);
    }
    else {
        return keys.reduce(function (acc, next) {
            var _a;
            if (next === '*') {
                return Array.from({ length: state['*'].length }).map(function (_, i) { return ({
                    value: initializeState(state['*'].value, initialState, currentPath + "." + i),
                }); });
            }
            else {
                return __assign(__assign({}, acc), (_a = {}, _a[next] = __assign(__assign({}, acc[next]), { value: initializeState(state[next], initialState, currentPath ? currentPath + "." + next : next) }), _a));
            }
        }, {});
    }
};
var setInEveryLeaf = function (state, newValue) {
    if (isArray(state)) {
        return state.map(function (s) { return ({
            value: setInEveryLeaf(s.value, newValue),
        }); });
    }
    else if (isObject(state)) {
        return Object.keys(state).reduce(function (acc, next) {
            var _a;
            return __assign(__assign({}, acc), (_a = {}, _a[next] = __assign(__assign({}, acc[next]), { value: setInEveryLeaf(state[next].value, newValue) }), _a));
        }, {});
    }
    else {
        return newValue;
    }
};
var setDeep = function (arr, state, newValue) {
    var _a;
    if (state === void 0) { state = {}; }
    if (arr.length == 0) {
        return __assign(__assign({}, state), { value: setInEveryLeaf(state.value, newValue) });
    }
    else if (arr[0] === "*") {
        return __assign(__assign({}, state), { value: (state.value || []).map(function (v) { return setDeep(arr.slice(1), v, newValue); }) });
    }
    else if (isNumber(arr[0])) {
        return __assign(__assign({}, state), { value: __spreadArrays((state.value || []).slice(0, parseInt(arr[0])), [
                setDeep(arr.slice(1), state.value[arr[0]], newValue)
            ], (state.value || []).slice(parseInt(arr[0]) + 1)) });
    }
    else {
        return __assign(__assign({}, state), { value: __assign(__assign({}, state.value), (_a = {}, _a[arr[0]] = setDeep(arr.slice(1), state.value[arr[0]], newValue), _a)) });
    }
};
export default (function (state, action) {
    var _a, _b;
    if (state === void 0) { state = { blueprint: {}, store: {}, validators: {} }; }
    var payload = action.payload;
    switch (action.type) {
        case MOUNT_MODEL: {
            var model = payload.model.split('.');
            return __assign(__assign({}, state), { blueprint: __assign(__assign({}, state.blueprint), constructBlueprint(model, state.blueprint)), validators: __assign(__assign({}, state.validators), (_a = {}, _a[payload.model] = payload.validator, _a)) });
        }
        case INITIALIZE_STATE:
            return __assign(__assign({}, state), { store: initializeState(state.blueprint, payload.initialState), validators: __assign(__assign({}, state.validators), payload.validators) });
        case SET_VALUE: {
            var model = payload.model.split('.');
            return __assign(__assign({}, state), { store: __assign(__assign({}, state.store), (_b = {}, _b[model[0]] = setDeep(model.slice(1), state.store[model[0]], payload.value), _b)) });
        }
        case MARK_VALIDATED: { // combine with show_errors
            return state;
            // const model = fromDotProp(payload.model);
            // return {
            //   ...state,
            //   [model]: {
            //     ...state[model],
            //     validated: true,
            //   },
            // };
        }
        case SHOW_ERRORS:
            return state;
        // return {
        //   ...Object.keys(state).reduce((nextState, next) => {
        //     const model = fromDotProp(next);
        //     return {
        //       ...nextState,
        //       [model]: {
        //         ...state[model],
        //         validated: true
        //       },
        //     };
        //   }, {}),
        // };
        default:
            return state;
    }
});
export var getError = function (state, model) {
    return getIn(state, fromDotProp(model), 'validation', model, 'errors');
};
var flattenObject = function (obj, result) {
    if (result === void 0) { result = []; }
    for (var i in obj) {
        if (isObject(obj[i])) {
            result = __spreadArrays(flattenObject(obj[i], result));
        }
        else {
            result = __spreadArrays(result, [obj[i]]);
        }
    }
    ;
    return result;
};
export var getValidators = function (validators, model, modelValidators) {
    if (modelValidators === void 0) { modelValidators = []; }
    var combinedValidators = modelValidators;
    if (validators[model]) {
        combinedValidators = __spreadArrays(modelValidators, [validators[model]]);
    }
    if (model && model.split('.').some(function (m) { return isNumber(m); })) {
        return getValidators(validators, model.replace(/\.\d+(?![\s\S]*\.\d+)/, '.*'), combinedValidators);
    }
    return combinedValidators;
};
export var getValue = function (state, arr) {
    if (arr.length == 0) {
        return state;
    }
    else if (!state || Object.keys(state).length === 0) {
        return undefined;
    }
    else if (arr[0] === "*") {
        var values_1 = flattenObject((state || []).map(function (v) { return getValue(v.value, arr.slice(1)); }));
        if (values_1.some(function (v) { return v !== values_1[0]; })) {
            return undefined;
        }
        else {
            return values_1[0];
        }
    }
    else {
        return getValue(getIn(state, arr[0], 'value'), arr.slice(1));
    }
};
// const parseValues = (value: Scalar, parseValue: {[key: string]: Function}, model: string) => {
//   if (Array.isArray(value)) {
//     return value.map((v, i) => {
//       const parse = parseValue['*'] || parseValue[i] || ((x: Scalar) => x);
//       return v ? parse(v) : undefined;
//     });
//   }
//
//   const key = getIndex(model);
//   const parse = parseValue[key] || ((x: Scalar) => x);
//   return value ? parse(value) : value;
// };
//
var getStateValue = function (state) {
    if (state === void 0) { state = {}; }
    if (isArray(state.value)) {
        return state.value.map(function (s) { return getStateValue(s); });
    }
    else if (isObject(state.value)) {
        return Object.keys(state.value).reduce(function (acc, v) {
            var _a;
            return (__assign(__assign({}, acc), (_a = {}, _a[v] = getStateValue(state.value[v]), _a)));
        }, {});
    }
    else {
        return state.value;
    }
};
export var getValues = function (store) {
    return Object.keys(store).reduce(function (acc, model) {
        var _a;
        return (__assign(__assign({}, acc), (_a = {}, _a[model] = getStateValue(store[model]), _a)));
    }, {});
};
export var hasBeenValidated = function (state, model) {
    return getIn(state, fromDotProp(model), 'validation', model, 'validated');
};
export var hasError = function (state, model) {
    var errors = getError(state, model);
    return Boolean(getIn(errors, 'length'));
};
export var hasErrors = function (state) {
    return Object.keys(state).filter(function (model) { return hasError(state, model); }).length > 0;
};
//# sourceMappingURL=control.js.map